 

-- FailBot w/ Support for : LDB-1.1, Sink-2.0, and configuration via AceOptions

-- Credits to FailBot by Veev for the original code used to make libfail.




local ADDON_NAME = "FailBoat"

local addon = LibStub("AceAddon-3.0"):NewAddon(ADDON_NAME, "LibSink-2.0", "AceConsole-3.0")

local L	= LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME)

local media = LibStub("LibSharedMedia-3.0") 
local fail = LibStub("LibFail-1.0")

local defaults = {
	profile = {
		sinkoptions = { --[["sink20OutputSink"] = "Popup"]] },
		sound = nil,
	}
}


--[===[@debug@ 
local function dbg(...)
    if Prat then
        Prat.CustomPrint(addon, nil, nil, nil, nil, nil, true, ...)
    else
        print(...)
    end
end
--@end-debug@]===]

function addon:SetValue(info, b)
	self.db.profile[info[#info]] = b
end
function addon:GetValue(info)
	return self.db.profile[info[#info]]
end


local options = {
    name = L["addon_name"],
    desc = L["addon_desc"],
    type = "group",
    get = "GetValue",
    set = "SetValue", 
    handler = addon,
    args = {
		output = addon:GetSinkAce3OptionsDataTable(),
	
		sound = {
            name = L["sound_name"],
            desc = L["sound_desc"],
 			type = "select",
			dialogControl = 'LSM30_Sound',
			values = AceGUIWidgetLSMlists.sound,
		},
    },
}

options.args.output.inline = true

local DO = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject(ADDON_NAME, {
		type = "data source",
		text = L["addon_name"],
        icon = [[Interface\ICONS\Spell_Arcane_MindMastery]],
})


function DO.OnTooltipShow(tip)
    tip:AddLine(L["left_click_desc"])

--[===[@debug@ 
    tip:AddLine("")
    tip:AddLine("Shift-Click To Test A Random Event")
--@end-debug@]===]

end



function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("FailBoatDB", defaults, "Default")

	self:SetSinkStorage(self.db.profile.sinkoptions)

	local acreg = LibStub("AceConfigRegistry-3.0")
	
	acreg:RegisterOptionsTable(ADDON_NAME, options)
	
	local acdia = LibStub("AceConfigDialog-3.0")
	
	acdia:AddToBlizOptions(ADDON_NAME, ADDON_NAME)
	
	addon:RegisterChatCommand(ADDON_NAME:lower(), function() acdia:Open(ADDON_NAME) end)
end


local fail_events = {
	"Fail_Sartheron_LavaWaves",
	"Fail_Heigan_Dance",
	"Fail_Any_VoidZone",
	"Fail_Sapphiron_Breath",
	"Fail_Frogger",
	"Fail_Thaddius_PolaritySwitch",
	"Fail_Thaddius_Jump",
}

local fail_counts = {}

function addon:OnEnable()
	for _, v in pairs(fail_events) do
		fail.RegisterCallback(self, v, "Fail")
        fail_counts[v] = {}
	end

	media.RegisterCallback(self, "LibSharedMedia_Registered", "SharedMedia_Registered")
	media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "SharedMedia_Registered")

    fail.RegisterCallback(self, "Fail_Active")
    fail.RegisterCallback(self, "Fail_Inactive")

    if fail.active then 
        self:Fail_Active()
    else
        self:Fail_Inactive()
    end        
end


function addon:Fail_Active()
   DO.text =  L["addon_name"]..L["addon_active"]
end

function addon:Fail_Inactive()
   DO.text =  L["addon_name"]..L["addon_inactive"]
end

function addon:OnDisable()
	fail.UnregisterAllEvents(self)
	media.UnregisterAllCallbacks(self)
end

local SOUND = media.MediaType.SOUND
local count

function addon:Fail(event, name, ...)
    count = (fail_counts[event][name] or 0) + 1
    fail_counts[event][name] = count


--[===[@debug@ 
    dbg(event, name, ...)
--@end-debug@]===]
	local txt = L[event]:format(name, ...)

    -- add a count to the end
    txt = txt..(L["fail_count"]):format(count)

--[===[@debug@ 
    dbg(txt)
--@end-debug@]===]

	if self.db.profile.sound then

		PlaySoundFile(media:Fetch(SOUND, self.db.profile.sound))
	end

	self:Pour(txt)
end


function addon:SharedMedia_Registered(event, mediatype, name)
    if mediatype == SOUND then
        -- Update list of sounds?
    end
end

function DO.OnClick(frame, button)
	if button == "RightButton" then
--		print("right-click")
	else
		if IsShiftKeyDown() then
--			print("shift-click")
--[===[@debug@ 
            fail:Test()
            DO.OnEnter(frame)
--@end-debug@]===]
		elseif IsAltKeyDown() then
            for event,counts in pairs(fail_counts) do
                wipe(counts)
            end
            print("Fail Counts Reset")
--			print("alt-click")
		else
            LibStub("AceConfigDialog-3.0"):Open(ADDON_NAME)
		end
	end
end



function DO.OnEnter(self)
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
    GameTooltip:ClearLines()
    DO.OnTooltipShow(GameTooltip)
    GameTooltip:Show()
end

function DO.OnLeave(self)
    GameTooltip:Hide()
end

function DO.OnTooltipShow(tooltip)   
	tooltip:AddLine(L["addon_name"],  1.0, 1.0, 0.4)
    for event,counts in pairs(fail_counts) do
    	tooltip:AddLine(L["label_"..event],  1.0, 1.0, 0.4)
        for name,count in pairs(counts) do
        	tooltip:AddLine("  "..name.."   "..tostring(count), 0.4, 0.4, 1.0)
        end
    end
end


--[===[@debug@ 
FB = addon
--@end-debug@]===]

